import 'package:freezed_annotation/freezed_annotation.dart';

part 'room_power_level.freezed.dart';
part 'room_power_level.g.dart';

/// Represents the power levels of a room.
///
/// The [RoomPowerLevel] class contains the following properties:
///
/// * [users]: A map of user IDs to power levels.
///
/// * [usersDefault]: The default power level for users in the room.
///
/// * [events]: A map of event type strings to power levels.
///
/// * [eventsDefault]: The default power level for events in the room.
///
/// * [stateDefault]: The default power level for room state events in the room.
///
/// * [ban]: The power level required to ban a user from the room.
///
/// * [kick]: The power level required to kick a user from the room.
///
/// * [redact]: The power level required to redact an event in the room.
///
/// * [invite]: The power level required to invite a user to the room.
///
/// * [historical]: The power level required to retrieve historical events in the room.
@Freezed(makeCollectionsUnmodifiable: false)
class RoomPowerLevel with _$RoomPowerLevel {
  factory RoomPowerLevel({
    @JsonKey(name: 'users') Map<String, dynamic>? users,
    @JsonKey(name: 'users_default') int? usersDefault,
    @JsonKey(name: 'events') Map<String, int?>? events,
    @JsonKey(name: 'events_default') int? eventsDefault,
    @JsonKey(name: 'state_default') int? stateDefault,
    @JsonKey(name: 'ban') int? ban,
    @JsonKey(name: 'kick') int? kick,
    @JsonKey(name: 'redact') int? redact,
    @JsonKey(name: 'invite') int? invite,
    @JsonKey(name: 'historical') int? historical,
  }) = _RoomPowerLevel;

  factory RoomPowerLevel.fromJson(Map<String, dynamic> json) =>
      _$RoomPowerLevelFromJson(json);

  factory RoomPowerLevel.initial() => RoomPowerLevel(
        users: <String, dynamic>{},
        usersDefault: 0,
        events: <String, int?>{
          'm.room.name': 50,
          'm.room.power_levels': 100,
          'm.room.history_visibility': 100,
          'm.room.canonical_alias': 50,
          'm.room.avatar': 50,
          'm.room.tombstone': 100,
          'm.room.server_acl': 100,
          'm.room.encryption': 100,
          'm.space.child': 50,
          'm.room.topic': 50,
          'm.room.pinned_events': 50,
          'm.reaction': 0,
          'm.room.redaction': 0,
          'org.matrix.msc3401.call': 50,
          'org.matrix.msc3401.call.member': 50,
          'im.vector.modular.widgets': 50,
          'io.element.voice_broadcast_info': 50
        },
        eventsDefault: 0,
        stateDefault: 50,
        ban: 50,
        kick: 50,
        redact: 50,
        invite: 50,
        historical: 100,
      );
}
